'''
Copyright (C) 2024 1P2D
https://1p2d.com
me@1p2d.com

Created by 1P2D - Felix Pierron

This file is part of the Select Mode Keymaps.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, see <https://www.gnu.org/licenses>.
'''

bl_info = {
    "name": "Select Mode Keymaps",
    "description": "Assign keymaps to toggle Object/Edit Mode and Select Modes",
    "author": "1P2D <me@1p2d.com>",
    "version": (1, 1, 0),
    "blender": (4, 2, 0),
    "category": "Interface",
    "location": "3D View > Toolbar",
    "doc_url": "https://www.1p2d.com",
}

import bpy
from bpy.types import Operator
from .preferences import SelectModeShortcutsPreferences  # Import preferences class

# Function to check if a modal tool is active
def is_modal_tool_active(context):
    active_tool = context.workspace.tools.from_space_view3d_mode(context.mode, create=False)
    # List of modal tool idnames to check against
    modal_tools = {
        'builtin.move', 
        'builtin.rotate', 
        'builtin.scale', 
        'builtin.proportional_edit', 
        'builtin.knife_tool',
        # Add other modal tools here as needed
    }
    return active_tool and active_tool.idname in modal_tools

# Custom Operator for toggling Object/Edit Mode and setting Select Mode
class OBJECT_OT_custom_editmode_toggle_set_select_mode(Operator):
    bl_idname = "object.editmode_toggle_selectmode"  # Unique bl_idname
    bl_label = "Set Select Mode"
    bl_options = {'REGISTER', 'UNDO'}
    
    select_mode: bpy.props.EnumProperty(
        name="Select Mode",
        description="Select mode to activate",
        items=[
            ('VERT', "Vertex", "Vertex Select Mode"),
            ('EDGE', "Edge", "Edge Select Mode"),
            ('FACE', "Face", "Face Select Mode"),
        ],
        default='EDGE',
    )
    
    def execute(self, context):
        # Prevent execution if a modal tool is active
        if is_modal_tool_active(context):
            self.report({'WARNING'}, "Cannot change select mode while a modal tool is active.")
            return {'CANCELLED'}
        
        if context.mode == 'OBJECT':
            # Switch to Edit Mode if in Object Mode and the active object is a mesh
            if context.active_object and context.active_object.type == 'MESH':
                bpy.ops.object.mode_set(mode='EDIT')
                # After switching to Edit Mode, set the desired selection mode
                bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type=self.select_mode)
            else:
                self.report({'WARNING'}, "Active object is not a mesh.")
                return {'CANCELLED'}
        elif context.mode == 'EDIT_MESH':
            # Only set the desired selection mode without toggling back to Object Mode
            bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type=self.select_mode)
        else:
            self.report({'WARNING'}, "Set Select Mode is only available in Object and Edit Mesh modes.")
            return {'CANCELLED'}
        
        return {'FINISHED'}

# Keymap Registration Functions
def register_keymap():
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        # 3D View Keymap
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
        
        # 1. Toggle between Object and Edit Mode with COMMA (',')
        km.keymap_items.new("object.editmode_toggle", 'COMMA', 'PRESS')
        
        # 2. Toggle and set Face Select Mode with CTRL + COMMA
        kmi_face = km.keymap_items.new(
            "object.editmode_toggle_selectmode", 
            'COMMA', 
            'PRESS', 
            ctrl=True
        )
        kmi_face.properties.select_mode = 'FACE'  # Set to FACE Select Mode
        
        # 3. Toggle and set Vertex Select Mode with ALT + COMMA
        kmi_vert = km.keymap_items.new(
            "object.editmode_toggle_selectmode", 
            'COMMA', 
            'PRESS', 
            alt=True
        )
        kmi_vert.properties.select_mode = 'VERT'  # Set to Vertex Select Mode
        
        # 4. Toggle and set Edge Select Mode with SHIFT + COMMA
        kmi_edge = km.keymap_items.new(
            "object.editmode_toggle_selectmode", 
            'COMMA', 
            'PRESS', 
            shift=True
        )
        kmi_edge.properties.select_mode = 'EDGE'  # Set to Edge Select Mode

def unregister_keymap():
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        km = kc.keymaps.get('3D View')
        if km:
            # Collect keymap items to remove
            items_to_remove = []
            for kmi in km.keymap_items:
                if kmi.idname in {
                    "object.editmode_toggle_selectmode",
                    "object.editmode_toggle",
                }:
                    items_to_remove.append(kmi)
            # Remove the collected keymap items
            for kmi in items_to_remove:
                km.keymap_items.remove(kmi)

# Register and Unregister Functions
def register():
    bpy.utils.register_class(OBJECT_OT_custom_editmode_toggle_set_select_mode)
    bpy.utils.register_class(SelectModeShortcutsPreferences)
    register_keymap()

def unregister():
    unregister_keymap()
    bpy.utils.unregister_class(SelectModeShortcutsPreferences)
    bpy.utils.unregister_class(OBJECT_OT_custom_editmode_toggle_set_select_mode)

if __name__ == "__main__":
    register()
